package ru.ifmo.qbeing.common;

import ru.ifmo.qbeing.QBeing;
import universum.bi.BeingInterface;
import universum.bi.Constants;
import universum.bi.Location;
import universum.util.Util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@SuppressWarnings("deprecation")
public class BeingUtil {
    public static Location getRandomNearest(BeingInterface bi, Location center, Collection<Location> points) {
        return getLocationFarestOrNearestRandom(bi, center, points, false);
    }

    public static Location getRandomFarest(BeingInterface bi, Location center, Collection<Location> points) {
        return getLocationFarestOrNearestRandom(bi, center, points, true);
    }

    public static Location getLocationEqual(BeingInterface bi, Location center, Collection<Location> points, Location locEqual) {
        float findDist = bi.distance(center, locEqual);
        Location perfectLoc = null;
        float minDiff = 0;
        boolean bFirst = true;
        for (Location loc : points) {
            float dist = bi.distance(center, loc);
            if (dist - findDist < minDiff || bFirst) {
                minDiff = dist - findDist;
                perfectLoc = loc;
            }
            bFirst = false;
        }
        return perfectLoc;
    }

    public static Location getLocationFarestOrNearestRandom(BeingInterface bi, Location center, Collection<Location> points, boolean farest) {
        boolean bFirst = true;
        float min = 0;
        float max = 0;
        List<Location> minLoc = new ArrayList<Location>();
        List<Location> maxLoc = new ArrayList<Location>();
        for (Location loc : points) {
            float dist = bi.distance(center, loc);
            if (dist > max || bFirst) {
                max = dist;
                maxLoc.clear();
                maxLoc.add(loc);
            } else {
                if (Math.abs(dist - max) < BehaviourConstants.EQUAL_DISTANCE) {
                    maxLoc.add(loc);
                }
            }
            if (dist < min || bFirst) {
                min = dist;
                minLoc.clear();
                minLoc.add(loc);
            } else {
                if (Math.abs(dist - min) < BehaviourConstants.EQUAL_DISTANCE) {
                    minLoc.add(loc);
                }
            }
            bFirst = false;
        }
        if (farest) {
            if (maxLoc.isEmpty()) {
                return null;
            }
            return maxLoc.get(Util.rnd(maxLoc.size()));
        } else {
            if (minLoc.isEmpty()) {
                return null;
            }
            return minLoc.get(Util.rnd(minLoc.size()));
        }
    }

    // - ftr: fake step count
    public static int getStepsCount(BeingInterface bi, Location loc1, Location loc2) {
        return (int) bi.distance(loc1, loc2);
    }

    public static float getEnergyToReach(QBeing me, BeingInterface bi, Location loc) {
        return (float) Math.ceil(bi.distance(bi.getLocation(me), loc) / me.getSpeed()) * Constants.K_movecost * me.getSpeed();
    }

    public static Location stepToward(BeingInterface bi, QBeing me, Location to) {
        Location from = bi.getLocation(me);
        float speed = me.getSpeed();
        int dx = to.getX() - from.getX();
        int dy = to.getY() - from.getY();
        int dirx = dx == 0 ? 0 : (dx > 0 ? 1 : -1);
        int diry = dy == 0 ? 0 : (dy > 0 ? 1 : -1);

        int ddx = Math.abs(dx);
        int ddy = Math.abs(dy);
        if (ddx > Constants.getWidth() - ddx) {
            ddx = Constants.getWidth() - ddx;
            dirx = -dirx;
        }
        if (ddy > Constants.getHeight() - ddy) {
            ddy = Constants.getHeight() - ddy;
            diry = -diry;
        }

        if (ddx <= speed && ddy <= speed) {
            return to;
        }

        int s = (int) speed;
        ddx = Math.min(ddx, s) * dirx;
        ddy = Math.min(ddy, s) * diry;

        int x = from.getX() + ddx;
        int y = from.getY() + ddy;
        if (x < 0)
            x += Constants.getWidth();
        if (x >= Constants.getWidth())
            x -= Constants.getWidth();
        if (y < 0)
            y += Constants.getHeight();
        if (y >= Constants.getHeight())
            y -= Constants.getHeight();

        return bi.createLocation(x, y);
    }
}
