package ru.ifmo.qbeing.global;

import ru.ifmo.qbeing.*;
import ru.ifmo.qbeing.common.BehaviourConstants;
import ru.ifmo.qbeing.common.LocationInfo;
import universum.bi.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

@SuppressWarnings("deprecation")
public class Colony implements Comparable<Colony> {
    private LocationInfo pi;
    private Set<QBeing> supliers;

    public void addSupplier(QBeing being) {
        supliers.add(being);
    }

    public boolean removeSupplier(QBeing being) {
        supliers.remove(being);
        return supliers.size() == 0;
    }

    public Colony(LocationInfo pi, QBeing being) {
        this.pi = pi;
        this.supliers = new HashSet<QBeing>();
        this.supliers.add(being);
    }

    public Set<QBeing> getSupliers() {
        return supliers;
    }

    public Event colonyAction(BeingInterface bi, QBeing me) {
        Event ev = null;
        float energy = bi.getPointInfo(me).getCount(me);
        float consumed = 0;
        ArrayList<QBeing> ts = new ArrayList<QBeing>(this.supliers);
        Collections.sort(ts);
        QBeing toGive = null;
        float giveEnergy = 0;

        if (GlobalStrategy.instance().heal) {
            for (int i = ts.size() - 1; i >= 0; i--) {
                if (consumed > energy) {
                    break;
                }
                QBeing being = ts.get(i);
                if (being.getEnergy() < BehaviourConstants.MIN_HEALTH * being.getMass()) {
                    if (me == being) {
                        break;
                    }
                    float energyRequired = being.getMass() * BehaviourConstants.FEED_NOT_TO_DIE;
                    if (toGive == null) {
                        toGive = being;
                        giveEnergy = energyRequired;
                    }
                    consumed += energyRequired;
                }
            }
        }
        if (toGive == null && ev == null)
            for (QBeing being : ts) {
                if (being.getState() == BeingState.S_PRODUCER) {
                    if (consumed > energy) {
                        break;
                    }
                    if (being == me && me.getEnergy() < Constants.K_toborn * me.getMass()) {
                        float give = Math.min(me.getMass() - me.getEnergy(), me.getMass() * Constants.K_bite);
                        updateSupplierEnergy(me, me.getEnergy() + give);

                        StateMachineContext ctx = new StateMachineContext(me, bi);
                        ctx.put("amount", give);
                        EventManager.handleBeingEvent(BeingEvent.E108, me, ctx);

                        ev = new Event(EventKind.ACTION_EAT, me.getMass() - me.getEnergy());
                        break;
                    }
                    float energyRequired = being.getMass() * Constants.K_toborn - being.getEnergy();
                    if (energyRequired >= BehaviourConstants.GIVE_TO_OTHER_IF_REQUIRED_LESS * me.getMass() && toGive == null) {
                        toGive = being;
                        giveEnergy = calcGiveMaxEnergy(me, bi, energyRequired);
                    }
                    consumed += Math.min(giveEnergy, being.getMass() * Constants.K_bite);
                }
            }
        if (ev == null && toGive != null) {
            if (bi.getEnergy(me) >= BehaviourConstants.MIN_ENERGY_TO_GIVE * me.getMass()) {
                float give = Math.min(giveEnergy, BehaviourConstants.MAX_GIVE_ENERGY * me.getMass());
                updateSupplierEnergy(toGive, toGive.getEnergy() + give);
                updateSupplierEnergy(me, me.getEnergy() - give);

                StateMachineContext ctx = new StateMachineContext(me, bi);
                ctx.put("amount", give);
                ctx.put("other", toGive.getId());
                EventManager.handleBeingEvent(BeingEvent.E109, me, ctx);

                ev = new Event(EventKind.ACTION_GIVE, toGive.getId(), give);
            }
        }
        return ev;
    }

    private float calcGiveMaxEnergy(QBeing me, BeingInterface bi, float energyRequired) {
        int turnsToLive = GlobalStrategy.instance().getMaxTurns() - bi.getTurnsCount();
        float energyForTurns = (Constants.K_masscost * me.getMass() * (turnsToLive + 1));
        float energyToGive = me.getEnergy() - energyForTurns;
        return Math.min(energyToGive, energyRequired);
    }


    public String toString() {
        return "[" + supliers.size() + "] (" + pi.location.toString() + ")";
    }

    public void updateSupplierEnergy(QBeing me, float energy) {
        me.setEnergy(energy);
    }

    public Location getLocation() {
        return pi.location;
    }

    public int compareTo(Colony o) {
        int s1 = getSupliers().size();
        int s2 = o.getSupliers().size();
        return s1 == s2 ? 0 : (s1 > s2 ? 0 : 1);
    }

    public boolean equals(Object obj) {
        return getLocation().equals(((Colony) obj).getLocation());
    }
}
