package ru.ifmo.qbeing.global;

import ru.ifmo.qbeing.QBeing;
import ru.ifmo.qbeing.common.LocationInfo;
import universum.bi.*;

import java.util.*;

public class ColonyControl {
    protected Map<Location, Colony> colonies;
    private List<Location> golden;

    public ColonyControl() {
        colonies = new HashMap<Location, Colony>();
    }

    public boolean setupColony(PointInfo pi, QBeing me) {
        Location location = pi.getLocation();
        if (colonies.containsKey(location)) {
            return false;
        }
        colonies.put(location, new Colony(new LocationInfo(pi, me), me));
        return true;
    }

    public void removeColonySupplier(Location loc, QBeing me) {
        Colony ci = colonies.get(loc);
        if (ci != null) {
            if (ci.removeSupplier(me)) {
                colonies.remove(ci.getLocation());
            }
        }
    }

    public void addColonySupplier(QBeing me, BeingInterface bi) {
        Colony ci = colonies.get(bi.getLocation(me));
        if (ci != null) {
            ci.addSupplier(me);
        } else {
            setupColony(bi.getPointInfo(me), me);
        }
    }

    public QBeing[] getSuppliers(Location loc) {
        Colony ci = colonies.get(loc);
        QBeing[] suppliers = null;
        if (ci != null) {
            suppliers = ci.getSupliers().toArray(new QBeing[1]);
        }
        return suppliers;
    }

    public Set<QBeing> getSuppliersList(Location loc) {
        Colony ci = colonies.get(loc);
        if (ci != null) {
            return ci.getSupliers();
        }
        return null;
    }

    public Event colonyAction(BeingInterface bi, QBeing me) {
        Colony colony = colonies.get(bi.getLocation(me));
        if (colony != null) {
            return colony.colonyAction(bi, me);
        } else {
            return null;
        }
    }

    public void updateupdateSupplierEnergy(QBeing me) {
        Colony ci = colonies.get(me.getLocation());
        if (ci != null) {
            ci.updateSupplierEnergy(me, me.getEnergy());
        }
    }

    public List<Location> getGoldenLocations() {
        if (golden == null) {
            golden = new ArrayList<Location>();
            calcGolden();
        }
        return golden;
    }

    private void calcGolden() {
        List<Colony> largestColonies = new ArrayList<Colony>();
        for (Colony c : colonies.values()) {
            if (!largestColonies.contains(c)) {
                largestColonies.add(c);
                Collections.sort(largestColonies);
                if (largestColonies.size() > Constants.NUM_GOLDEN) {
                    largestColonies.remove(largestColonies.size() - 1);
                }
            }
        }
        for (Colony c : largestColonies) {
            golden.add(c.getLocation());
        }
    }
}
